<?php
/**
 * @file
 * Contains \Drupal\webtexttool\Controller\WebtexttoolController.
 */

namespace Drupal\webtexttool\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Component\Serialization\Json;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use Drupal\user\PrivateTempStoreFactory;

class WebtexttoolController extends ControllerBase {

  const WEBTEXTTOOL_URL = 'https://app.webtexttool.com';
  const WEBTEXTOOL_PATH = '/WTT.Service/api';


  /**
   * The temp store factory.
   *
   * @var \Drupal\user\PrivateTempStoreFactory
   */
    protected $tempStoreFactory;

  public function content() {
    return array(
      '#type' => 'markup',
      '#markup' => $this->webtexttoolLogin('/user/IsAuthenticated'),
    );
  }

  /**
   * Helper function to check if user is authenticated.
   */
  public function webtexttoolIsAuthenticated() {
    $is_authenticated = $this->webtexttoolConnect('/user/IsAuthenticated');
    return $is_authenticated;
  }
  /**
   * Implements searchkeyword from api.
   */
  public function webtexttoolSearchKeyword($keyword, $language = 'us') {
    $response = $this->webtexttoolConnect('/keyword/searchkeyword/' . $keyword . '/' . $language);
    return $response;
  }

  /**
   * Implements GetSuggestions from api.
   */
  public function webtexttoolAnalyse($html, $keywords, $language = 'en') {

    // Set default paramaters
    $parameters = array(
      'content' => $html,
      'keywords' => $keywords,
      'languageCode' => 'en',
    );
    $response = $this->webtexttoolConnect('/suggestion/GetSuggestions','', 'post', $parameters);
    return $response;
  }


    protected function webtexttoolConnect($function, $token = '',$method = 'get', $parameters = array()) {

    if($token == ''){
      $token = $this->webtexttoolLogin();
    }
    try {
      $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal', 'Authorization' => 'bearer ' . $token);
      $client = new Client([
        'base_uri' => SELF::WEBTEXTTOOL_URL,
        'headers' => $headers
      ]);

      if($method == 'post'){
        // Posting incoming paramaters
        $request = $client->post(SELF::WEBTEXTOOL_PATH . $function, [
          'json' => $parameters,
      ]);

      }
      else{
        $request = $client->get(SELF::WEBTEXTOOL_PATH . $function);
      }

      $data = (string) $request->getBody();
      if (empty($data)) {
        return FALSE;
      }
      else{
        $data = json_decode($data);
        if(!empty($data)){
          return $data;
        }
        else{
          return false;
        }
      }
    }
    catch (RequestException $e) {
      drupal_set_message(t('Unable to connect to the webtexttool service please check your credentials and try again later.'), 'error');
      return FALSE;
    }

    return false;
  }

  /**
   * Register to webtexttool api service.
   *
   */
  public function webtexttoolRegister($email, $pass, $language) {
    $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal');
    $data_array = array(
      'Language' => $language,
      'UserName' => $email,
      'Password' => $pass,
    );
    try {

      $client = new Client([
        'base_uri' => SELF::WEBTEXTTOOL_URL,
      ]);
      $request = $client->post(SELF::WEBTEXTOOL_PATH . "/user/register", [
        'headers' => [$headers],
        'json' => $data_array,
      ]);
      $data = (string) $request->getBody();
      if (empty($data)) {
        return FALSE;
      }
      else{
        $data = json_decode($data);
        $token = $data->access_token;
        if(!empty($token)){
          $this->webtexttoolSetToken($token);
          return $token;
        }
        else{
          return false;
        }
      }
    }
    catch (RequestException $e) {
      return FALSE;
    }
  }

  /**
   * Logs in to webtexttool api service.
   *
   */
  protected function webtexttoolLogin() {

    $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal');
    $data_array = array(
      'Language' => \Drupal::config('webtexttool.settings')->get('language'),
      'RememberMe' => TRUE,
      'UserName' => \Drupal::config('webtexttool.settings')->get('user'),
      'Password' => \Drupal::config('webtexttool.settings')->get('pass'),
    );
    try {

      $client = new Client([
        'base_uri' => SELF::WEBTEXTTOOL_URL,
      ]);
      $request = $client->post(SELF::WEBTEXTOOL_PATH . "/user/login", [
        'headers' => [$headers],
        'json' => $data_array,
      ]);
      $data = (string) $request->getBody();
      if (empty($data)) {
        return FALSE;
      }
      else{
        $data = json_decode($data);
        $token = $data->access_token;
        if(!empty($token)){
          $this->webtexttoolSetToken($token);
          return $token;
        }
        else{
          drupal_set_message(t('Unable to login to the webtexttool service please check your credentials and try again later.'), 'error');
          return false;
        }
      }

    }
    catch (RequestException $e) {
      drupal_set_message(t('Unable to login to the webtexttool service please check your credentials and try again later.'), 'error');
      return FALSE;
    }
  }

  /**
   * Generate the correct headers with bearer.
   *
   */
  protected function webtexttoolHeaders($token = '') {
    $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal');
    // Get the token
    $token = $this->webtexttoolGetToken($token);
    if($token && $token !== ''){
      $headers['Authorization'] = 'bearer ' . $token;
    }
    return $headers;
  }

  /**
   * Set token from webtexttool.
   */
  protected function webtexttoolSetToken($token) {
    $tempstore = \Drupal::service('user.private_tempstore')->get('mymodule_name');
    $tempstore->set('webtexttooltoken', $token);
  }

  /**
   * Get token from webtexttool.
   */
  protected function webtexttoolGetToken($token = '') {
    if($token !== ''){
      return $token;
    }
    $tempstore = \Drupal::service('user.private_tempstore')->get('mymodule_name');
    $token = $tempstore->get('webtexttooltoken');
    if(empty($token)){
      $token = $this->webtexttoolLogin();
    }

    return $token;
  }

  /**
   * Implements getkeywordsources from api.
   */
  public function webtexttoolGetSources() {

    if ($cache = \Drupal::cache()->get('webtexttool_getkeywordsources') ) {
      $keywords = $cache->data;
    }
    else {
      $keywords = array();
      // Get the list of languages supported.
      $keywords_languages = $this->webtexttoolConnect('/keyword/getkeywordsources');
      if ($keywords_languages) {
        foreach ($keywords_languages as $keyword) {
          $keywords[$keyword->Value] = $keyword->Name;
        }
        $set_cache = \Drupal::cache()->set('webtexttool_getkeywordsources', $keywords);
      }
    }

    return $keywords;
  }
}